/**
 * description
 *
 * @author eson
 *2022年7月21日-13:48:01
 */
package com.yuandian.dataflow.controller;

import java.io.Serializable;
import java.time.Instant;
import java.util.ArrayList;

import com.alipay.sofa.jraft.Status;
import com.alipay.sofa.jraft.entity.PeerId;
import com.alipay.sofa.jraft.error.RemotingException;
import com.alipay.sofa.jraft.rpc.InvokeCallback;
import com.alipay.sofa.jraft.rpc.RpcContext;
import com.alipay.sofa.jraft.rpc.RpcProcessor;
import com.google.protobuf.Any;
import com.yuandian.dataflow.statemachine.StateServerFactory;
import com.yuandian.dataflow.statemachine.operate.Operate;
import com.yuandian.dataflow.statemachine.operate.Operate.OperateType;
import com.yuandian.dataflow.statemachine.OperateClosure;
import com.yuandian.dataflow.statemachine.rpc.RaftResponse;
import com.yuandian.dataflow.statemachine.rpc.OperateProcessor;
 
import com.yuandian.dataflow.statemachine.rpc.annotations.ProcessorRaft;
import com.yuandian.dataflow.statemachine.state.State;

import lombok.Getter;
import lombok.Setter;
import lombok.var;
import lombok.extern.slf4j.Slf4j;

/**
 * description
 *
 * @author eson
 *2022年7月21日-13:48:01
 */

@Slf4j
@ProcessorRaft
public class PacketsProcessor implements RpcProcessor<PacketsProcessor.PacketsRequest> {

    @Setter
    @Getter
    public static class PacketsRequest implements Serializable {
        private ArrayList<Any> packets = new ArrayList<>();
    }

    @Override
    public void handleRequest(RpcContext rpcCtx, PacketsRequest request) {
            // StateServerFactory.getStateServer().updateFsmStateAsync(s, onCompleted);
            
            var resp = new RaftResponse();
            resp.setMsg(rpcCtx.getRemoteAddress());
            resp.setSuccess(true);
 
            var ss = StateServerFactory.getStateServer();
 
            log.info("{} handler request.packets.size(): {}", ss.getNode().getNodeId().getPeerId() ,request.packets.size());
            ss.useFsmStateNotLock((state)->{
                
                var work = state.getWorkers().get( ss.getCluster().getServerId());
 
              
                work.setTaskQueueSize( work.getTaskQueueSize() - request.packets.size());
                work.setUpdateAt(Instant.now());
                log.debug("workerState taskQueueSize: {} psize: {}", work.getTaskQueueSize(), request.packets.size());
                var op = new Operate(OperateType.PUT);
                op.setValue(work);

                Operate.CallOperate(op, new OperateClosure() {

                    @Override
                    public void run(Status status) {
                        log.info("{}", this.getResponse());
                        var resp = new RaftResponse();
                        resp.setRedirect(StateServerFactory.getStateServer().getNode().getLeaderId());
                        rpcCtx.sendResponse(resp);
                    }
                    
                });
            });
            

    }

    @Override
    public String interest() {
        return PacketsRequest.class.getName();
    }
}
