package com.yuandian.dataflow.statemachine.operate;

import java.io.Serializable;

import com.alipay.sofa.jraft.Status;
import com.alipay.sofa.jraft.error.RemotingException;
import com.alipay.sofa.jraft.rpc.InvokeCallback;
import com.yuandian.dataflow.projo.Response;
import com.yuandian.dataflow.statemachine.OperateClosure;
import com.yuandian.dataflow.statemachine.StateServerFactory;
import com.yuandian.dataflow.statemachine.rpc.OperateProcessor;
import com.yuandian.dataflow.statemachine.rpc.RaftResponse;
import com.yuandian.dataflow.statemachine.state.StateFactory;

 
import lombok.Data;
import lombok.var;
import lombok.extern.slf4j.Slf4j;

/**
 * 操作
 * 
 * @author eson
 */
@Slf4j
@Data
@var
public class Operate implements Serializable {

    public static enum OperateType {
        PUT, REMOVE;
    }

    private OperateType type;
    private Object value;

    public Operate(OperateType t) {
        this.type = t;
    }

    public <T> T getValue() {
        return (T) this.value;
    };

    public <T> void setValue(T value) {
        this.value = value;
        return;
    };

    public static void CallOperate(Operate op, OperateClosure closure) {
        
        var ss = StateServerFactory.getStateServer();
        if (StateServerFactory.getStateServer().isLeader()) {
            ss.applyOperate(op, closure);
            return;
        }

        var request = new OperateProcessor.RequestOperate();
        request.setOperate(op);

        var leaderId = StateServerFactory.getStateServer().getNode().getLeaderId();
        try {
            ss.getRpcClient().invokeAsync(leaderId.getEndpoint(),
                    request, new InvokeCallback() {

                        @Override
                        public void complete(Object result, Throwable err) {
                            log.info("{}", result);
                            var resp = (RaftResponse)result;
                            closure.setResponse(resp);
                            closure.success(resp.getOperate());
                        }

                    }, 5000);
        } catch (InterruptedException | RemotingException e) {
            // TODO Auto-generated catch block
            closure.failure("failure", null);
           log.info("{}", e.toString());
        }

    }
}
