/**
 * description
 *
 * @author eson
 *2022年7月20日-10:00:05
 */
package com.yuandian.dataflow.statemachine.state;

import java.time.Instant;
import java.util.ArrayList;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import com.alibaba.nacos.shaded.io.grpc.netty.shaded.io.netty.handler.codec.http.HttpContentEncoder.Result;
import com.alipay.sofa.jraft.Status;
import com.alipay.sofa.jraft.entity.PeerId;
import com.alipay.sofa.jraft.error.RemotingException;
import com.alipay.sofa.jraft.rpc.InvokeCallback;
import com.google.protobuf.Any;
import com.yuandian.dataflow.controller.PacketsProcessor.PacketsRequest;
import com.yuandian.dataflow.proto.msgtype.BacktrackingFlowOuterClass;
import com.yuandian.dataflow.statemachine.StateServerFactory;
import com.yuandian.dataflow.statemachine.operate.Operate;
import com.yuandian.dataflow.statemachine.operate.Operate.OperateType;
import com.yuandian.dataflow.statemachine.OperateClosure;

import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import lombok.var;
import lombok.extern.slf4j.Slf4j;

/**
 * description
 *
 * @author eson
 *         2022年7月20日-10:00:05
 */
@Slf4j
@Getter
@Setter
@ToString
public class StateFactory {

    @Getter
    @Setter
    @ToString
    public static class PeerIdCap {
        private PeerId peer;
        private long cap;

        public PeerIdCap(PeerId pid, long cap) {
            this.peer = pid;
            this.cap = cap;
        }
    }

    public static Thread masterExecute = new Thread(new Runnable() {
        @Override
        public void run() {
            try {
                while (true) {
                    log.debug("master execute {}", StateServerFactory.getServerId());
                    var alivePeers = StateServerFactory.getCluster().getRaftNode().listAlivePeers();
                    log.debug("master execute {}", StateServerFactory.getCluster().getRaftNode().listAlivePeers());
                    if (alivePeers != null) {

                        var ss = StateServerFactory.getStateServer();
                        // var state = ss.getFsm().getState();

                        ss.useFsmStateNotLock((state) -> {
                            alivePeers.forEach((peer) -> {
                                WorkerState ws = state.getWorkers().get(peer);
                                if (ws != null) {
                                    var cap = 100 - ws.getTaskQueueSize();
                                    log.debug("cap :{} peer: {}", cap, peer);
                                    if (cap <= 0) {
                                        return;
                                    }
                                    ws.setUpdateAt(Instant.now());
                                    ws.setTaskQueueSize(100);

                                    var request = new PacketsRequest();
                                    for (int i = 0; i < cap; i++) {
                                        var p = Any.pack(
                                                BacktrackingFlowOuterClass.BacktrackingFlow.newBuilder()
                                                        .setTableId(10086)
                                                        .build());
                                        request.getPackets().add(p);
                                    }

                                    var op = new Operate(OperateType.PUT);
                                    op.setValue(ws);
                                    Operate.CallOperate(op, new OperateClosure() {
                                        @Override
                                        public void run(Status status) {

                                            log.info("{}", status);

                                            try {
                                                ss.getRpcClient().invokeAsync(peer.getEndpoint(),
                                                        request, new InvokeCallback() {

                                                            @Override
                                                            public void complete(Object result, Throwable err) {
                                                                log.info("{}", result);
                                                            }

                                                        }, 5000);
                                            } catch (InterruptedException | RemotingException e) {
                                                log.info("error send packets {}", e.toString());
                                            }
                                        }
                                    });

                                }
                            });

                        });

                        // ss.applyState(state, new SyncClosure<State>() {
                        // public void run(Status status) {
                        // log.debug("{}", status);
                        // };
                        // });

                    }

                    Thread.sleep(5000);
                }
            } catch (InterruptedException e) {
                log.info("{}", e.toString());
            }

        }
    });

    public static Thread getMasterExecute() {
        return masterExecute;
    }

    public static void Init() {

    }

}
